#pragma once

#include "libtoken_global.h"
#include <QList>
#include <QString>
#include <memory>

namespace lib_token {
	/**
	 * @brief Դʵ
	 */
	class IWordBase {
	public:
		/**
		 * @brief Դļ·
		 * @return
		 */
		virtual QString file() const = 0;
		/**
		* @brief ȡλ
		* @return ʼλ
		*/
		virtual uint64_t position() const = 0;
		/**
		 * @brief 
		 * @return
		 */
		virtual QString content() const = 0;
		/**
		 * @brief Դк
		 * @return
		 */
		virtual int row() const = 0;
		/**
		 * @brief Դк
		 * @return
		 */
		virtual int column() const = 0;
		/**
		 * @brief һ.
		 * 
		 * \return 
		 */
		virtual std::shared_ptr<const IWordBase> nextWord() const = 0;
	};

	/**
	 * @brief δԭʼ
	 */
	class WordContent : public IWordBase {
	private:
		int row_n, col_n;
		uint64_t doc_offset;
		QString text_n, path_p;

	public:
		WordContent(int r, int c, uint64_t pos, const QString& t, const QString& p);

		// WordBase interface
	public:
		virtual QString file() const override;
		virtual uint64_t position() const override;
		virtual QString content() const override;
		virtual int row() const override;
		virtual int column() const override;
		virtual std::shared_ptr<const IWordBase> nextWord() const override;
	};

	class LIBTOKEN_EXPORT WordImpl : public IWordBase {
	private:
		std::shared_ptr<const IWordBase> content_ptr;
		std::shared_ptr<const IWordBase> next_ptr;

	public:
		WordImpl(std::shared_ptr<const IWordBase> content, std::shared_ptr<const IWordBase> next);

		// ͨ IWordBase ̳
		QString file() const override;
		virtual uint64_t position() const override;
		QString content() const override;
		int row() const override;
		int column() const override;
		std::shared_ptr<const IWordBase> nextWord() const override;
	};

	/**
	 * @brief 쳣
	 */
	class LIBTOKEN_EXPORT TokenException {
	private:
		QString msg_store;

	public:
		TokenException(const QString& message);
		virtual QString message() const;
	};
	class ITokenDefine;
	/**
	 * @brief token
	 */
	class IToken : public IWordBase {
	public:
		/**
		 * @brief tokenƹ
		 * @return
		 */
		virtual std::shared_ptr<const ITokenDefine> define() const = 0;
	};

	/**
	 * @brief tokenƶ
	 */
	class ITokenDefine {
	public:
		/**
		 * @brief 
		 * @return
		 */
		virtual QString reviseWords() const = 0;
		/**
		 * @brief ͱʶ.
		 * 
		 * \return 
		 */
		virtual int typeMark() const = 0;
		/**
		 * @brief ƹؼ
		 * @return
		 */
		virtual QString regex() const = 0;

		/**
		 * @brief ﴦ
		 * @param content 
		 * @return tuple<token/null,remains> 
		 */
		virtual std::tuple<std::shared_ptr<const IToken>, std::shared_ptr<const IWordBase>> analysis(std::shared_ptr<const IWordBase> content) const = 0;
	};



	/**
	 * @brief ʷȡ
	 */
	class LIBTOKEN_EXPORT WordReader {
	private:
		QList<std::shared_ptr<const IWordBase>> parse_line(uint64_t start_pos, int row, const QString& line_text, const QString& path) const;
		QList<std::shared_ptr<const IWordBase>> extract_from(const QString& path) const;

	public:
		std::shared_ptr<const IWordBase> wordsFrom(const QString &path) const;
	};

} // namespace lib_token